package net.vtstar.zhongtong.avi.gateway.assembly.job;

import lombok.extern.slf4j.Slf4j;
import net.vtstar.scada.base.equipmgt.domain.Enum.EquipStatusEnum;
import net.vtstar.scada.base.gather.domain.EquipmentData;
import net.vtstar.scada.base.gather.job.GatherJob;
import net.vtstar.zhongtong.avi.gateway.assembly.domain.FloorLeatherHotMachine;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.springframework.stereotype.Component;

import java.util.Date;

/**
 * 地板革热熔涂胶设备数据采集JOB
 */
@Slf4j
@Component
public class FloorLeatherHotMachineJob extends GatherJob {

    private static final String NAME = "FLOOR_LEATHER_HOT";

    @Override
    protected void executeInternal(JobExecutionContext context) throws JobExecutionException {

        log.info("JobName: {}", context.getJobDetail().getKey().getName());
        super.gatherByModbus(NAME, FloorLeatherHotMachine.class);
    }

    @Override
    public boolean handleObject(EquipmentData equipmentData) {
        FloorLeatherHotMachine data = (FloorLeatherHotMachine) equipmentData;
        Integer state = data.getStatus();
        if (state != null && state == 0) {
            return false;
        }
        if (data.getCurrentYield() == 0){
            return false;
        }
        equipmentData.setEquipStatus(EquipStatusEnum.RUN);
        return true;
    }

    @Override
    public Object handleValue(String fieldName, Object value) {
        return value;
    }

    @Override
    protected void postHandle(EquipmentData equipmentData) {
        ((FloorLeatherHotMachine)equipmentData).setCreateTime(new Date());
    }
}
