package net.vtstar.zhongtong.avi.gateway.painting.job;

import lombok.extern.slf4j.Slf4j;
import net.vtstar.scada.base.equipmgt.domain.Enum.EquipStatusEnum;
import net.vtstar.scada.base.gather.domain.EquipmentData;
import net.vtstar.scada.base.gather.job.GatherJob;
import net.vtstar.zhongtong.avi.gateway.painting.domain.PaintingRobotMachine;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.springframework.stereotype.Component;

import java.util.Date;

/**
 * 涂装机器人数据采集JOB
 */
@Slf4j
@Component
public class PaintingRobotMachineJob extends GatherJob {

    private static final String NAME = "PAINTING_ROBOT";

    @Override
    protected void executeInternal(JobExecutionContext context) throws JobExecutionException {

        log.info("JobName: {}", context.getJobDetail().getKey().getName());
        super.gatherByModbus(NAME, PaintingRobotMachine.class);
    }

    @Override
    public boolean handleObject(EquipmentData equipmentData) {
        PaintingRobotMachine data = (PaintingRobotMachine) equipmentData;
        Integer state = data.getStatus();
        if (state != null && state == 0) {
            return false;
        }
        if(data.getRunState() == 1){
            equipmentData.setEquipStatus(EquipStatusEnum.RUN);
        }else{
            equipmentData.setEquipStatus(EquipStatusEnum.ALARM);
        }

        return true;
    }

    @Override
    public Object handleValue(String fieldName, Object value) {
        Integer valueInt = Integer.valueOf(value.toString());
        return (valueInt & 0x01);
    }

    @Override
    protected void postHandle(EquipmentData equipmentData) {
        ((PaintingRobotMachine)equipmentData).setCreateTime(new Date());
    }
}
