package net.vtstar.zhongtong.avi.gateway.welding.job;

import lombok.extern.slf4j.Slf4j;
import net.vtstar.scada.base.equipmgt.domain.Enum.EquipStatusEnum;
import net.vtstar.scada.base.gather.domain.EquipmentData;
import net.vtstar.scada.base.gather.job.GatherJob;
import net.vtstar.zhongtong.avi.gateway.welding.domain.PlateChainMachine;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.springframework.stereotype.Component;

import java.util.Date;

/**
 * 焊装板链数据采集JOB
 */
@Slf4j
@Component
public class PlateChainMachineJob extends GatherJob {

    private static final String NAME = "PLATE_CHAIN";

    @Override
    protected void executeInternal(JobExecutionContext context) throws JobExecutionException {

        log.info("JobName: {}", context.getJobDetail().getKey().getName());
        super.gatherByModbus(NAME, PlateChainMachine.class);
    }

    @Override
    public boolean handleObject(EquipmentData equipmentData) {
        PlateChainMachine data = (PlateChainMachine) equipmentData;
        Integer state = data.getStatus();
        if (state != null && state == 0) {
            return false;
        }
        equipmentData.setEquipStatus(EquipStatusEnum.RUN);
        return true;
    }

    @Override
    public Object handleValue(String fieldName, Object value) {
        Integer valueInt = Integer.valueOf(value.toString());
        return (valueInt & 0x01);
    }

    @Override
    protected void postHandle(EquipmentData equipmentData) {
        ((PlateChainMachine)equipmentData).setCreateTime(new Date());
    }
}
